@@ -2,8 +2,8 @@
 """
 ingest.py: source client for Voctomix.
 
-   Copyright: 2015,2016,2017 Carl F. Karsten <carl@nextdayvideo.com>,
-                             Ryan Verner <ryan@nextdayvideo.com.au>
+   Copyright: 2015-2020 Carl F. Karsten <carl@nextdayvideo.com>,
+                        Ryan Verner <ryan@nextdayvideo.com.au>
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
@@ -43,12 +43,34 @@ def mk_video_src(args, videocaps):
     # args come from command line
     # videocaps come from voctocore
 
-    if args.video_source == 'dv':
+    if args.video_source == 'test':
+
+        video_src = """
+    videotestsrc name=videosrc {attribs}
+    """
+        # things to render as text ontop of test video
+        video_src += """ !
+    clockoverlay
+        text="Source: {hostname}\nCaps: {videocaps}\nAttribs: {attribs}\n"
+        halignment=left line-alignment=left
+            """.format(hostname=socket.gethostname(),
+                    videocaps=videocaps,
+                    attribs=args.video_attribs)
+
+    elif args.video_source == 'spacescope':
+        # Stereo visualizer
+        # pair up with audoi test beep for a handy AV sync test.
+        video_src = """
+      audio_tee. ! queue !
+    spacescope shader=none style=lines {attribs}
+           """
+
+    elif args.video_source == 'dv':
         video_src = """
             dv1394src name=videosrc {attribs} !
         dvdemux name=demux !
         queue max-size-time=4000000000 !
-        dvdec !
+        dvdec
             """
 
     elif args.video_source == 'hdv':
@@ -56,86 +78,84 @@ def mk_video_src(args, videocaps):
             hdv1394src {attribs} name=videosrc !
         tsdemux !
         queue max-size-time=4000000000 !
-        decodebin !
+        decodebin
             """
 
-    elif args.video_source == 'hdmi2usb':
-        # https://hdmi2usb.tv
-        # Note: this code works with 720p
+    elif args.video_source in ('hdmi2usb', 'uvc-mjpeg'):
         video_src = """
-            v4l2src {attribs} name=videosrc !
+            v4l2src {attribs} name=videosrc
+        """
+        video_src += """ !
                 queue max-size-time=4000000000 !
-        image/jpeg,width=1280,height=720 !
-                jpegdec !
-            """
+        image/jpeg,{caps} !
+                jpegdec
+            """.format(caps=extract_caps(videocaps))
+
+    elif args.video_source == 'uvc-raw':
+
+        video_src = """
+            v4l2src {attribs} name=videosrc
+        """
+        video_src += """ !
+        video/x-raw,{caps}
+        """.format(
+            caps=extract_caps(videocaps))
 
     elif args.video_source == 'ximage':
+        # Doesn't work with Wayland.
         # startx=0 starty=0 endx=1919 endy=1079 !
         video_src = """
             ximagesrc {attribs} name=videosrc
-                   use-damage=false !
+                   use-damage=false
             """
 
     elif args.video_source == 'blackmagic':
         video_src = """
-            decklinkvideosrc name=videosrc {attribs} !
+            decklinkvideosrc name=videosrc {attribs}
             """
-        # yadif !
-        # deinterlace
 
     elif args.video_source == 'png':
         video_src = """
             multifilesrc {attribs}
                 caps="image/png" !
-            pngdec !
+            pngdec
             """
+        # tip: --video-attribs "loop=true"
+        # tip: --video-attribs "location=sc1204a.png"
 
     elif args.video_source == 'file':
         video_src = """
             multifilesrc {attribs} !
             decodebin name=src
             src. !
-            queue !
+            queue
             """
 
-    elif args.video_source == 'test':
-
+    elif args.video_source == 'rtmp':
         video_src = """
-    videotestsrc name=videosrc {attribs} !
-    """
-        # things to render as text ontop of test video
-        video_src += """
-    clockoverlay
-        text="Source: {hostname}\nCaps: {videocaps}\nAttribs: {attribs}\n"
-        halignment=left line-alignment=left !
-            """.format(hostname=socket.gethostname(),
-                    videocaps=videocaps,
-                    attribs=args.video_attribs)
+            rtmpsrc {attribs} !
+            decodebin name=src
+            src. !
+            queue
+            """
+    else:
+        # What I wish I had done from the start.
+        # similar to args.src, only just for the video part.
+        video_src = args.video_source
 
-    elif args.video_source == 'spacescope':
-        # Stereo visualizer
-        # pair up with test beep for a handy AV sync test.
-        video_src = """
-      audio_tee. ! queue !
-    spacescope shader=none style=lines {attribs} !
-           """
 
     if args.monitor:
-        if args.debug:
-            videosink="fpsdisplaysink"
-        else:
-            videosink="autovideosink"
 
-        video_src += """
+        video_src += f""" !
             tee name=t ! queue !
-                    videoconvert ! {videosink} sync=false
-                    t. ! queue !
-            """.format(videosink=videosink)
+                    videoconvert ! {args.monitor} sync=false
+                    t. ! queue
+            """
 
     if args.video_elements:
-        video_src += args.video_elements + " !\n"
+        video_src += f" ! {args.video_elements}\n"
 
-    video_src += videocaps + " !\n"
+    video_src += f" ! {videocaps}\n"
 
     video_src = video_src.format(attribs=args.video_attribs)
 
@@ -151,57 +171,70 @@ def mk_audio_src(args, audiocaps):
         'audiocaps': audiocaps,
     }
 
-    if args.audio_source in ['dv', 'hdv']:
+    if args.audio_source == 'test':
+        audio_src = """
+            audiotestsrc wave=ticks freq=330 {attribs} name=audiosrc
+            """
+
+    elif args.audio_source in ['dv', 'hdv']:
         # this only works if video is from DV also.
         # or some gst source that gets demux ed
         audio_src = """
             demux.audio !
-                queue !
+                queue
                 """
 
-    elif args.audio_source == 'file':
+    elif args.audio_source in ('file', 'rtmp'):
         # this only works if video is from ...
         # some gst source that gets demux ed, I guess.
         audio_src = """
         src. !
-                queue !
+                queue
                 """
 
     elif args.audio_source == 'pulse':
         audio_src = """
                 pulsesrc {attribs} {base_audio_attribs} name=audiosrc !
-                queue max-size-time=4000000000 ! audiorate !
+                queue max-size-time=4000000000
                 """
 
     elif args.audio_source == 'alsa':
         audio_src = """
                 alsasrc {attribs} {base_audio_attribs} name=audiosrc !
-                queue max-size-time=4000000000 ! audiorate !
+                queue max-size-time=4000000000
                 """
 
     elif args.audio_source == 'blackmagic':
         audio_src = """
-            decklinkaudiosrc name=audiosrc {attribs} !
+            decklinkaudiosrc name=audiosrc {attribs}
             """
+    else:
+        # What I wish I had done from the start.
+        # similar to args.src, only just for the audio part.
+        audio_src = args.audio_source
 
-    elif args.audio_source == 'test':
-        audio_src = """
-            audiotestsrc wave=ticks freq=330 {attribs} name=audiosrc !
-            """
 
     if args.audio_elements:
-        audio_src += args.audio_elements + " !\n"
+        audio_src += f" ! {args.audio_elements}\n"
+
 
     audio_src += """
-       {audiocaps} !
-    tee name=audio_tee
-      audio_tee. ! queue !
+    !   {audiocaps}
+    ! tee name=audio_tee
+      audio_tee. ! queue
       """
     audio_src = audio_src.format(**d)
 
     return audio_src
 
 
+def extract_caps(videocaps, caps=("width", "height")):
+    """Filter out caps (list of keys) from a videocaps strings"""
+    parts = videocaps.split(',')
+    filtered = [cap for cap in parts if cap.split('=')[0] in caps]
+    return ','.join(filtered)
+
+
 def mk_client(core_ip, port):
 
     client = "tcpclientsink host={host} port={port}".format(
@@ -220,17 +253,17 @@ def mk_pipeline(args, server_caps, core_ip):
         audio_src = mk_audio_src(args, server_caps['audiocaps'])
         src =  """
             {video_src}
-             mux.
+             ! mux.
             {audio_src}
-             mux.
-            matroskamux name=mux !
+             ! mux.
+            matroskamux name=mux
         """.format(video_src=video_src, audio_src=audio_src)
 
     client = mk_client(core_ip, args.port)
 
     pipeline = """
             {src}
-    {client}
+    ! {client}
     """.format(src=src, client=client)
 
     # remove blank lines to make it more human readable
@@ -341,24 +374,25 @@ def run_pipeline(pipeline, clock, audio_delay=0, video_delay=0):
     senderPipeline.bus.connect("message::eos", on_eos)
     senderPipeline.bus.connect("message::error", on_error)
 
-    print("playing...")
+    print("ingest.py: streaming to core...")
     senderPipeline.set_state(Gst.State.PLAYING)
 
     mainloop = GLib.MainLoop()
     try:
         mainloop.run()
     except KeyboardInterrupt:
-        print('Terminated via Ctrl-C')
+        print('ingest.py: Terminated via Ctrl-C')
 
-    print('Shutting down...')
+    print('ingest.py: Shutting down...')
     senderPipeline.set_state(Gst.State.NULL)
-    print('Done.')
+    print('ingest.py: Done.')
 
     return
 
 
 def get_args():
     parser = argparse.ArgumentParser(
+            formatter_class=argparse.ArgumentDefaultsHelpFormatter,
             description='''Vocto-ingest Client with Net-time support.
             Gst caps are retrieved from the server.
             Run without parameters: send test av to localhost:10000
@@ -377,9 +411,6 @@ def get_args():
 
     parser.add_argument(
         '--video-source', action='store',
-        choices=[
-            'dv', 'hdv', 'udp_h264', 'hdmi2usb', 'blackmagic',
-            'ximage', 'png', 'file', 'test', 'spacescope'],
         default='test',
         help="Where to get video from")
 
@@ -395,13 +426,11 @@ def get_args():
 
     parser.add_argument(
         '--video-elements', action='store',
-        default='videoconvert ! videorate ! videoscale',
-        help="gst video elments ! after src")
+        default='videoconvert ! deinterlace ! videorate ! videoscale',
+        help="gst video elments after src.")
 
     parser.add_argument(
         '--audio-source', action='store',
-        choices=['dv', 'hdv', 'file',
-            'alsa', 'pulse', 'blackmagic', 'test', ],
         default='test',
         help="Where to get audio from")
 
@@ -419,11 +448,12 @@ def get_args():
     parser.add_argument(
         '--audio-elements', action='store',
         default="audioconvert ! audioresample ! audiorate",
-        help="gst audio elments ! after src")
+        help="gst audio elments after src.")
 
     parser.add_argument(
-        '-m', '--monitor', action='store_true',
-        help="local display sink")
+        '-m', '--monitor',
+        # action='store_true',
+        help="local display sink (autovideosink, fbdevsink, fpsdisplaysink...")
 
     parser.add_argument(
         '--host', action='store',
@@ -449,7 +479,7 @@ def get_args():
 
 
 def main():
-    GObject.threads_init()
+
     Gst.init([])
 
     args = get_args()
